#include <GL/glut.h>
#include <GL/glu.h>
#include <GL/gl.h>
#include <sys/time.h>
#include <stdio.h>
#include <string>

#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600


void gpgpuUpdate();
void myIdle();
void myDisplay();
float getTime();

void initGL(int argc, char **argv, const char *name) {
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
	glutInitWindowPosition(-1, -1);
	glutCreateWindow((std::string("GPGPU abstraction framework demo") + name).c_str());

	glutDisplayFunc(myDisplay);
	glutIdleFunc(myIdle);
	
	glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	float aspX = (float)WINDOW_WIDTH*0.001f;
	float aspY = (float)WINDOW_HEIGHT*0.001f;
	glFrustum(-aspX, aspX, -aspY, aspY, 1.0, 3.0);
	glTranslatef(0.0, 0.0, -2.0); 
}


float rotation[3];
void myIdle() {
	float time = getTime();
	rotation[0] = 30.0f*time;
	rotation[1] = 20.0f*time;
	rotation[2] = 50.0f*time;

	gpgpuUpdate();
	glutPostRedisplay();
}


void myDisplay() {
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glEnable(GL_DEPTH_TEST);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glRotatef(rotation[0], 1.0, 0.0, 0.0);
	glRotatef(rotation[1], 0.0, 1.0, 0.0);
	glRotatef(rotation[2], 0.0, 0.0, 1.0);

	glBegin(GL_QUADS);
		glColor3f(0.6, 0.6, 1.0); // Light blue
		glNormal3f(0.0, 0.0, 1.0);

		glTexCoord2f(0.0, 0.0);
		glVertex3f(-0.5, -0.5, 0.5);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(-0.5, 0.5, 0.5);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(0.5, 0.5, 0.5);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.5, -0.5, 0.5);

		glColor3f(1.0, 0.6, 0.6); // Light red
		glNormal3f(0.0, 0.0, -1.0);

		glTexCoord2f(0.0, 0.0);
		glVertex3f(-0.5, -0.5, -0.5);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(-0.5, 0.5, -0.5);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(0.5, 0.5, -0.5);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.5, -0.5, -0.5);

		glColor3f(0.6, 1.0, 0.6); // Light green
		glNormal3f(1.0, 0.0, 0.0);

		glTexCoord2f(0.0, 0.0);
		glVertex3f(0.5, -0.5, -0.5);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(0.5, -0.5, 0.5);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(0.5, 0.5, 0.5);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.5, 0.5, -0.5);

		glColor3f(0.6, 1.0, 1.0); // Light cyan
		glNormal3f(-1.0, 0.0, 0.0);

		glTexCoord2f(0.0, 0.0);
		glVertex3f(-0.5, -0.5, -0.5);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(-0.5, -0.5, 0.5);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(-0.5, 0.5, 0.5);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(-0.5, 0.5, -0.5);

		glColor3f(1.0, 1.0, 0.6); // Light yellow
		glNormal3f(0.0, 1.0, 0.0);

		glTexCoord2f(0.0, 0.0);
		glVertex3f(-0.5, 0.5, -0.5);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(-0.5, 0.5, 0.5);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(0.5, 0.5, 0.5);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.5, 0.5, -0.5);

		glColor3f(1.0, 0.6, 1.0); // Light purple
		glNormal3f(0.0, -1.0, 0.0);

		glTexCoord2f(0.0, 0.0);
		glVertex3f(-0.5, -0.5, -0.5);
		glTexCoord2f(1.0, 0.0);
		glVertex3f(-0.5, -0.5, 0.5);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(0.5, -0.5, 0.5);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(0.5, -0.5, -0.5);
	glEnd();

	glFlush();
	glutSwapBuffers();
}


float getTime() {
	struct timeval tv;
	gettimeofday(&tv, NULL);
	static double startTime = 0.0;
	if (startTime == 0.0)
		startTime = (double)tv.tv_sec + (double)tv.tv_usec/1e6;

	return ((double)tv.tv_sec + (double)tv.tv_usec/1e6)-startTime;
}
